﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Microsoft.Office.Interop.Visio;

// HadoopNode.cs
// Code for node class that will be the main building block of the internal representation of the Hadoop cluster.

namespace HadoopVisProto
{
    class HadoopNode
    {
        private int type;
        public string name;
        private float hdSpaceUsed;
        private float mapSlotsUsed;
        private float redSlotsUsed;
        public bool dataActive = false;
        public bool taskActive = false;

        protected bool dataNode = false;
        protected bool taskTracker = false;

        public void markAsTaskTracker() {taskTracker = true;}
        public void markAsDataNode() {dataNode = true;}

        private HadoopNode hdfsParent;
        private HadoopNode mapredParent;
        private List<HadoopNode> children;
        private bool _updated = false;
        public bool updated
        {
            get { return _updated; }
            set { _updated = value; }
        }

        // Hadoop tree methods for building internal representation of the cluster

        public HadoopNode(int type, string name)
        {
            this.type = type;
            this.name = name;

            if (type < 2) {
                children = new List<HadoopNode>();
            }
        }

        public void setHDSpace(float percentRemaining)
        {
            hdSpaceUsed = 100 - percentRemaining;
        }
        public double getHDSpace()
        {
            return hdSpaceUsed;
        }

        public void setMapUse(int mapSlots, int mapSlotsInUse)
        {
            mapSlotsUsed = 100*mapSlotsInUse/mapSlots;
        }
        public void setReduceUse(int reduceSlots, int reduceSlotsInUse)
        {
            redSlotsUsed = 100*reduceSlotsInUse/reduceSlots;
        }
        public float getMapUse()
        {
            return mapSlotsUsed;
        }
        public float getReduceUse()
        {
            return redSlotsUsed;
        }
        
        public void setHDFSParent(HadoopNode p) {hdfsParent = p;}
        public HadoopNode getHDFSParent() {return hdfsParent;}

        public void setMapredParent(HadoopNode p) { mapredParent = p; }
        public HadoopNode getMapredParent() { return mapredParent; }

        public bool addChild(HadoopNode newChild)
        {
            if (children.Exists(c => c.name.Equals(newChild.name)))
            {
                return false;
            } 
            else 
            {
                children.Add(newChild);
                return true;
            }

        }

        // Shape methods for the Visio side of things

        private Shape visShape;

        public Shape setShape(Shape newShape)
        {
            visShape = newShape;
            return visShape;
        }
        public Shape getShape()
        {
            return visShape;
        }
    }
}
